@@ -24,6 +24,7 @@ import shutil
 import stat
 
 from lbinstall.Model import IGNORED_PACKAGES
+from lbinstall.Model import VersionedObject
 from lbinstall.InstallAreaManager import InstallArea
 from lbinstall.PackageManager import PackageManager
 from lbinstall.extra.RPMExtractor import sanitycheckrpmfile
@@ -335,8 +336,13 @@ class Installer(object):
                                                     None)
             for local_package in local_packages:
                 if local_package.name == package.name:
-                    if (package.version > local_package.version or
-                        (package.version == local_package.version and
+                    cmpVers = VersionedObject.cmpStandardVersion(
+                        VersionedObject.getStandardVersion(package.version),
+                        VersionedObject.getStandardVersion(
+                            local_package.version))
+
+                    if (cmpVers > 0 or
+                        (cmpVers == 0 and
                          (package.release > local_package.release))):
                         raise Exception('Please use update! The script '
                                         'needs to install %s-%s-%s, but the '
@@ -347,8 +353,8 @@ class Installer(object):
                                                        local_package.name,
                                                        local_package.version,
                                                        local_package.release))
-                    if (package.version < local_package.version or
-                        (package.version == local_package.version and
+                    if (cmpVers < 0 or
+                        (cmpVers == 0 and
                          (package.release < local_package.release))):
                         raise Exception('You are downgrading! Please remove '
                                         'first the existing version! The  '
@@ -360,7 +366,7 @@ class Installer(object):
                                                        local_package.name,
                                                        local_package.version,
                                                        local_package.release))
-                    if (package.version == local_package.version and (
+                    if (cmpVers == 0 and (
                             package.release == local_package.release)):
                         raise Exception('Please use reinstall! The script '
                                         'needs to install %s-%s-%s, but the '
@@ -736,14 +742,14 @@ class Installer(object):
                     nothing_added = False
                     install_graph.add_edge([p, match])
                     if match not in extrapackages:
-                        install_graph, missing_tmp = self._getPackagesToInstall(
+                        install_graph, missingTmp = self._getPackagesToInstall(
                             match, extrapackages,
                             ignoreInstalled=ignoreInstalled,
                             packagelist=packagelist,
                             install_graph=install_graph,
                             nodeps_update=nodeps_update,
                             ignoredInstalledList=ignoredInstalledList)
-                        missing_deps.extend(missing_tmp)
+                        missing_deps.extend(missingTmp)
                 elif self.strict:
                     missing_deps.append({
                         'dependency': req,
@@ -1131,8 +1137,13 @@ class Installer(object):
                                                     None)
             for local_package in local_packages:
                 if local_package.name == package[0]:
-                    if (package[1] < local_package.version or
-                        (package[1] == local_package.version and
+                    cmpVers = VersionedObject.cmpStandardVersion(
+                        VersionedObject.getStandardVersion(package[1]),
+                        VersionedObject.getStandardVersion(
+                            local_package.version))
+
+                    if (cmpVers < 0 or
+                        (cmpVers == 0 and
                          (package[2] < local_package.release))):
                         raise Exception('You are downgrading! Please remove '
                                         'first the existing version! The '
@@ -1144,7 +1155,7 @@ class Installer(object):
                                                        local_package.name,
                                                        local_package.version,
                                                        local_package.release))
-                    if (package[1] == local_package.version and (
+                    if (cmpVers == 0 and (
                             package[2] == local_package.release)):
                         raise Exception('Please use reinstall! The script '
                                         'needs to install %s-%s-%s, but the '
@@ -1189,8 +1200,12 @@ class Installer(object):
                                                     None)
             for local_package in local_packages:
                 if local_package.name == package[0]:
-                    if (package[1] < local_package.version or
-                        (package[1] == local_package.version and
+                    cmpVers = VersionedObject.cmpStandardVersion(
+                        VersionedObject.getStandardVersion(package[1]),
+                        VersionedObject.getStandardVersion(
+                            local_package.version))
+                    if (cmpVers < 0 or
+                        (cmpVers == 0 and
                          (package[2] < local_package.release))):
                         raise Exception('You are downgrading! Please remove '
                                         'first the existing version! The '
@@ -1202,8 +1217,8 @@ class Installer(object):
                                                        local_package.name,
                                                        local_package.version,
                                                        local_package.release))
-                    if (package[1] > local_package.version or
-                        (package[1] == local_package.version and
+                    if (cmpVers > 0 or
+                        (cmpVers == 0 and
                          (package[2] > local_package.release))):
                         raise Exception('Please use upgrading! The script '
                                         'needs to install %s-%s-%s, but the '
@@ -1290,7 +1305,27 @@ class Installer(object):
                                                      releaseregexp))):
             print(l.rpmName())
 
-    def checkPackgesFromTuples(self, rpms_to_verify):
+    # Providers query
+    # ############################################################################
+    def queryProviders(self, nameregexp=None,
+                       versionregexp=None,
+                       releaseregexp=None):
+        ''' Implement the listing of provides
+
+        :param nameRegexp: Regex for the name of the package
+        :param versionRegexp: Regex for the version of the package
+        :param releaseRegexp: Regex for the release of the package
+        '''
+        allpackages = set()
+        for l in sorted(list(self.remoteListProvides(nameregexp))):
+            for p in self.remoteFindPackage(l.name,
+                                            l.version,
+                                            l.release):
+                allpackages.add(p)
+        for l in sorted(allpackages):
+            print(l.rpmName())
+
+    def checkPackagesFromTuples(self, rpms_to_verify):
         ''' Implement the verifiy method of packages
 
         :param rpms_to_verify: the list of rpms to be verified
@@ -1318,10 +1353,11 @@ class Installer(object):
             max_len_n = len(n) if len(n) > max_len_n else max_len_n
             max_len_v = len(v) if len(v) > max_len_v else max_len_v
             max_len_r = len(r) if len(r) > max_len_r else max_len_r
-
+        to_return = []
         # Sort first by the location ( local or remote path) and then by name
         for n, v, r, source in sorted(allpackages, key=lambda x: (x[3], x[0])):
             name = "%s" % n + ' '*(max_len_n - len(n))
             version = "%s" % v + ' '*(max_len_v - len(v))
             release = "%s" % r + ' '*(max_len_r - len(r))
-            print("%s\t%s\t%s\t%s" % (name, version, release, source))
+            to_return.append((name, version, release, source))
+        return to_return
