@@ -96,12 +96,14 @@ class LbInstallClient(object):
         self.prog = prog
         self.dry_run = None
         self.justdb = None
+        self.query_provides = False
+        self.is_regex = False
         self.overwrite = None
         self.tmp_dir = None
         self.pool_size = 5
 
         parser = LbInstallOptionParser(usage=usage(self.prog))
-        #parser.disable_interspersed_args()
+        # parser.disable_interspersed_args()
         parser.add_option('-d', '--debug',
                           dest="debug",
                           default=False,
@@ -217,6 +219,18 @@ class LbInstallClient(object):
                           default='output',
                           action="store",
                           help="The output filename for dot file")
+        parser.add_option('--with-regex',
+                          dest="is_regex",
+                          default=False,
+                          action="store_true",
+                          help="Allows regular expressions in the "
+                               "packages names")
+        parser.add_option('--provides',
+                          dest="query_provides",
+                          default=False,
+                          action="store_true",
+                          help="Queries the packages that provides the"
+                               " argument")
         self.parser = parser
 
     def main(self):
@@ -273,7 +287,14 @@ class LbInstallClient(object):
                 self.withdeps = True
             else:
                 self.withdeps = False
-
+            if opts.query_provides:
+                self.query_provides = True
+            else:
+                self.query_provides = False
+            if opts.is_regex:
+                self.is_regex = True
+            else:
+                self.is_regex = False
             if opts.tree_mode:
                 self.tree_mode = True
             else:
@@ -355,10 +376,22 @@ class LbInstallClient(object):
         # Now executing the command
         if mode == LbInstallClient.MODE_QUERY:
             # Mode that list packages according to a regexp
-            self.installer.queryPackages(*args[1:])
+            if not self.is_regex:
+                new_args = []
+                for arg in args[1:]:
+                    arg = re.escape(arg)
+                    new_args.append(arg)
+                args = [args[0]]
+                args.extend(new_args)
+            if self.query_provides:
+                self.installer.queryProviders(*args[1:])
+            else:
+                self.installer.queryPackages(*args[1:])
         elif mode == LbInstallClient.MODE_LIST:
             # Mode that list packages according to a regexp
-            self.installer.listInstalledPackages(* args[1:])
+            results = self.installer.listInstalledPackages(* args[1:])
+            for (name, version, release, source) in results:
+                print("%s\t%s\t%s\t%s" % (name, version, release, source))
         elif (mode == LbInstallClient.MODE_RM or
               mode == LbInstallClient.MODE_INSTALL or
               mode == LbInstallClient.MODE_UPDATE or
@@ -396,7 +429,7 @@ class LbInstallClient(object):
                 for rpmname, version, release in rpms_list:
                     self.log.info("Verifing installed packages : %s %s %s"
                                   % (rpmname, version, release))
-                self.installer.checkPackgesFromTuples(rpms_list)
+                self.installer.checkPackagesFromTuples(rpms_list)
             elif mode == LbInstallClient.MODE_RM:
                 for rpmname, version, release in rpms_list:
                     self.log.info("Removing RPM: %s %s %s"
