@@ -31,7 +31,7 @@ from lbinstall.extra.ThreadPoolManager import ThreadPool
 from lbinstall.Graph import Graph
 from lbinstall.DependencyManager import IGNORED_PACKAGES
 from lbinstall.Model import Requires
-
+import re
 
 # Little util to find a file in a directory tree
 def findFileInDir(basename, dirname):
@@ -200,8 +200,8 @@ class YumChecker(object):
             missing_deps.extend(missing_deps_tmp)
         return missing_deps
 
-    def checkPackgesFromTuples(self, rpms_to_install, csv_name=None,
-                               force_json=False):
+    def checkPackagesFromTuples(self, rpms_to_install, csv_name=None,
+                                force_json=False):
         """
         Gets the package list that will be installed including (optional) their
         dependencies
@@ -216,6 +216,32 @@ class YumChecker(object):
         self.display_missing_deps(missing_deps, export_file=csv_name,
                                   force_json=force_json)
 
+    def show_links(self, rpms_to_install, filter=None):
+        packagelist = self.getPackageListFromTuples(rpms_to_install)
+        if not packagelist:
+            raise Exception("Please specify one or more packages")
+
+        # Looking for the files to install
+        rpmtoinstall = list()
+        install_graph = None
+        missing_deps = []
+        for p in packagelist:
+            install_graph, missing_deps_tmp = self._getPackagesToInstall(
+                p, packagelist=packagelist,
+                extrapackages=set(),
+                install_graph=install_graph)
+            missing_deps.extend(missing_deps_tmp)
+        to_return = install_graph.links()
+        if filter:
+            to_return = []
+            filter = re.compile(filter)
+            for name, link in install_graph.links():
+                if filter.search(link):
+                    to_return.append((name, link))
+        from pprint import pprint
+        pprint(missing_deps)
+        return [url for _, url in to_return]
+
     def _getPackagesToInstall(self, p,
                               extrapackages=set(),
                               packagelist=None,
@@ -245,7 +271,7 @@ class YumChecker(object):
                 req, force_local=force_local)
             if match:
                 nothing_added = False
-                install_graph.add_edge([p, match])
+                install_graph.add_edge([p, match], link=match.url())
                 if match not in extrapackages:
                     install_graph, missing_tmp = self._getPackagesToInstall(
                         match, extrapackages,
